<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

use sly\Assets\Util;

class WV8_Authorisation_Renderer extends WV42_Renderer_Base {
	private $i      = 1;
	private $tokens = null;
	private $dest   = null;
	private $role   = null;

	/**
	 * render form
	 *
	 * @param  sly_Form        $form
	 * @param  pac_Role        $role
	 * @param  pac_Destination $destination
	 * @param  array           $destConfig
	 * @return sly_Form
	 */
	public function render(sly_Form $form, pac_Role $role, pac_Destination $destination, array $destConfig) {
		$container = sly_Core::getContainer();

		// make sure we don't have to call our database a billion times
		$this->buildCache($role, $destination);

		// prepare tokens
		$tokens = $destConfig['token'];
		$this->tokens = array_keys($tokens);
		$this->dest   = $destination;
		$this->role   = $role;

		foreach ($tokens as $token => $title) {
			$tokens[$token] = sly_translate($title);
		}

		// build tree
		$tree = $this->renderPages($this->getValues(), $container['wv8-provider']);

		// wrap in container
		$cont = new sly_Form_Container();
		$cont->addFormRowClass('rbac-article-tree');
		$cont->setContent($tree);

		// add assets
		$layout = $container['sly-layout'];
		$layout->addCSSFile(Util::addOnUri('webvariants/rbac-treeview', 'backend.less'));
		$layout->addJavaScriptFile(Util::addOnUri('webvariants/rbac-treeview', 'backend.js'));

		// include magic (unused since we have only one token, but keep it in case we add some later)
//		WV42_DestinationFormHelper::writeSuperDeclaration($destination, $destConfig);

		// add element
		$form->beginFieldset(t('wv8_title'), 'wv8_settings');
		return $form->add($cont);
	}

	public function getValues() {
		$container = sly_Core::getContainer();

		return $container['wv8-provider']->getPages();
	}

	protected function renderPages(array $pages, WV8_Settings_Provider $provider) {
		$ul = '';

		array_unshift($pages, sly_Authorisation_ListProvider::ALL);

		foreach ($pages as $page) {
			$isAll   = $page === sly_Authorisation_ListProvider::ALL;
			$classes = $this->getClasses(1, $isAll);
			$label   = $isAll ? t('all') : sly_translate($page, false);
			$id      = $isAll ? 0 : WV8_Util::getPageID($page);

			if ($isAll) {
				$classes[] = 'is-all';
			}

			// render sub tree
			$groups  = $isAll ? array() : $provider->getGroups(null, $page);
			$subTree = $this->renderGroups($page, $groups, $provider);

			// add item
			$ul .= $this->getListItem('page', $id, $classes, $label, $subTree);
		}

		if (!empty($ul)) {
			$ul = "<ul>$ul</ul>";
		}

		return $ul;
	}

	protected function renderGroups($page, array $groups, WV8_Settings_Provider $provider) {
		$ul = '';

		foreach ($groups as $group) {
			$classes = $this->getClasses(2, false);
			$label   = sly_translate($group, false);
			$id      = WV8_Util::getGroupID($page, $group);

			// render sub tree
			$settings = $provider->getSettings(null, $page, $group);
			$subTree  = $this->renderSettings($page, $group, $settings);

			// add item
			$ul .= $this->getListItem('group', $id, $classes, $label, $subTree);
		}

		if (!empty($ul)) {
			$ul = "<ul>$ul</ul>";
		}

		return $ul;
	}

	protected function renderSettings($page, $group, array $settings) {
		$ul = '';

		foreach ($settings as $setting) {
			$classes = $this->getClasses(3, true);
			$label   = sly_translate($setting->getTitle(), false);
			$id      = $setting->getID();
			$ul     .= $this->getListItem('setting', $id, $classes, $label, '');
		}

		if (!empty($ul)) {
			$ul = "<ul>$ul</ul>";
		}

		return $ul;
	}

	protected function getClasses($lvl, $leaf) {
		$classes = array('lvl-'.$lvl, $this->i % 2 ? 'odd' : 'even');

		if ($leaf) {
			$classes[] = 'leaf';
		}

		++$this->i;

		return $classes;
	}

	protected function getListItem($type, $id, $classes, $label, $subTree) {
		static $format =
		'<li class="%classes%">
			<div class="sly-select-checkbox-list">
				<input type="checkbox" name="wv42_perm[%dstid%][%type%-%id%][]" value="edit" id="wv42_perm_wv8_%type%_%id%" %checked-edit%class="chk-token-edit" />
				<label for="wv42_perm_wv8_%type%_%id%" class="sly-inline"> %label%</label>
				<span class="chk inherit"><a href="#" class="inheriter" title="%inherit%">&nbsp;</a></span>
			</div>
			%subtree%
		</li>';

		if (is_array($classes)) {
			$classes = implode(' ', $classes);
		}

		$item = str_replace(
			array('%classes%', '%dstid%', '%type%', '%id%', '%label%', '%subtree%', '%inherit%'),
			array($classes, $this->dest->getId(), $type, $id, sly_html($label), $subTree, ht('rbac_tv_click_to_inherit_rights')),
			$format
		);

		$id = $type.'-'.$id;

		foreach ($this->tokens as $token) {
			$replacement = isset($this->dataCache[$id]) && in_array($token, $this->dataCache[$id]) ? 'checked="checked" ' : '';
			$item        = str_replace('%checked-'.$token.'%', $replacement, $item);
		}

		return $item;
	}
}
