<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

class WV8_Settings_UserProvider extends WV8_Settings_Provider {
	protected $user;

	public function __construct(WV8_Settings_Manager $manager, sly_Model_User $user) {
		parent::__construct($manager);

		$this->user  = $user;
		$this->perms = array();
	}

	public function clearPermissionCache() {
		$this->perms = array();
	}

	public function canAccessSetting(WV8_Setting $setting) {
		return
			$this->user->isAdmin() ||
			$this->hasPermission('page-'.$setting->getPageID()) ||
			$this->hasPermission('group-'.$setting->getGroupID()) ||
			$this->hasPermission('setting-'.$setting->getID())
		;
	}

	protected function hasPermission($nodeID) {
		if (!isset($this->perms[$nodeID])) {
			$this->perms[$nodeID] = $this->user->hasPermission('global_settings_settings', 'edit', $nodeID);
		}

		return $this->perms[$nodeID];
	}

	public function getAllSettings() {
		$all    = parent::getAllSettings();
		$result = array();

		foreach ($all as $namespace => $settings) {
			foreach ($settings as $key => $setting) {
				if ($this->canAccessSetting($setting)) {
					$result[$namespace][$key] = $setting;
				}
			}
		}

		return $result;
	}
}
