<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

use wv\BabelCache\CacheInterface;

abstract class WV8_Util {
	/**
	 * get setting provider
	 *
	 * @return WV8_Settings_Provider
	 */
	public static function getProvider() {
		return sly_Core::getContainer()->get('wv8-provider');
	}

	/**
	 * get setting manager
	 *
	 * @return WV8_Settings_Manager
	 */
	public static function getManager() {
		return sly_Core::getContainer()->get('wv8-manager');
	}

	public static function translate($string) {
		preg_match_all('#translate:([a-z0-9_-]+)#i', $string, $matches, PREG_SET_ORDER);

		foreach ($matches as $match) {
			$string = str_replace($match[0], t($match[1]), $string);
		}

		return $string;
	}

	public static function translateAndSortList($list) {
		foreach ($list as $key => $value) {
			$list[$key] = self::translate($value);
		}

		uasort($list, 'strnatcasecmp');
		return $list;
	}

	public static function renderPartialForm($namespace) {
		self::prepareHead();

		$provider = self::getProvider();
		$groups   = $provider->getGroups($namespace);

		include _WV8_PATH.'templates/partial.phtml';
	}

	public static function prepareHead(sly_Layout $layout = null) {
		$layout   = $layout ? $layout : sly_Core::getLayout();
		$devUtils = 'webvariants/developer-utils';
		$wv8      = 'webvariants/global-settings';

		$layout->addJavaScriptFile(Util::addOnUri($devUtils, 'js/jquery.wvtogglefieldset.min.js'));
		$layout->addJavaScriptFile(Util::addOnUri($devUtils, 'js/jquery.cookie.min.js'));
		$layout->addJavaScriptFile(Util::addOnUri($wv8, 'js/globalsettings.js'));
		$layout->addCSSFile(Util::addOnUri($wv8, 'css/globalsettings.less'));
	}

	public static function fixValues($values, array $languageIDs = null) {
		$values = sly_makeArray($values);

		if ($languageIDs === null) {
			$languageIDs = sly_Util_Language::findAll(true);
		}

		// missing values?
		foreach ($languageIDs as $clangID) {
			if (!isset($values[$clangID])) {
				$values[$clangID] = null;
			}
		}

		// too many values?
		foreach (array_keys($values) as $clangID) {
			if (!in_array($clangID, $languageIDs)) {
				unset($values[$clangID]);
			}
		}

		return $values;
	}

	public static function clearCache(CacheInterface $cache = null) {
		$cache = $cache ? $cache : sly_Core::cache();
		$cache->clear('globalsettings', true);
	}

	public static function hash($str) {
		return substr(sha1($str), 0, 8);
	}

	public static function encodeValues($values) {
		return json_encode($values);
	}

	public static function decodeValues($values) {
		return json_decode($values, true);
	}

	public static function getPageID($pagename) {
		return self::hash($pagename);
	}

	public static function getGroupID($pagename, $group) {
		return self::hash(json_encode(array($pagename, $group)));
	}

	public static function getSettingID(WV8_Setting $setting) {
		return self::hash($setting->getNamespace().'/'.$setting->getName());
	}

	public static function comparePages($pageA, $pageB) {
		return $pageA === $pageB || self::getPageID($pageA) === $pageB;
	}

	public static function compareGroups($pageA, $groupA, $groupB) {
		return $groupA === $groupB || self::getGroupID($pageA, $groupA) === $groupB;
	}
}
