<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

use sly\Assets\Util;

class sly_Controller_Globalsettings extends sly_Controller_Backend implements sly_Controller_Interface {
	private $init;

	protected function init() {
		if ($this->init) return;
		$this->init = true;

		// get layout
		$layout = $this->getContainer()->getLayout();

		// inject CSS (JS is added by the form itself)
		$layout->addCSSFile(Util::addOnUri('webvariants/global-settings', 'css/globalsettings.less'));

		// print page title
		$layout->pageHeader(t('wv8_title'));
	}

	public function indexAction() {
		$this->init();

		$container = $this->getContainer();
		$provider  = $this->getUserBoundProvider();
		$pages     = $provider->getPages();

		if (empty($pages)) {
			$this->render('intro.phtml', array(), false);
		}
		else {
			$this->render('overview.phtml', array('provider' => $provider), false);
		}
	}

	public function checkPermission($action) {
		$user = $this->getCurrentUser();
		return $user && ($user->isAdmin() || $user->hasPermission('pages', 'globalsettings'));
	}

	protected function getViewFolder() {
		return _WV8_PATH.'/templates/';
	}

	protected function getUserBoundProvider() {
		$user    = $this->getCurrentUser();
		$manager = $this->getContainer()->get('wv8-manager');

		return new WV8_Settings_UserProvider($manager, $user);
	}

	protected function getForm() {
		$provider  = $this->getUserBoundProvider();
		$container = $this->getContainer();
		$router    = $container->getApplication()->getRouter();
		$layout    = $container->getLayout();

		return new WV8_Form($provider, $router, $layout);
	}
}
