<?php
/*
 * Copyright (c) 2013, webvariants GbR, http://www.webvariants.de
 *
 * This file is released under the terms of the MIT license. You can find the
 * complete text in the attached LICENSE file or online at:
 *
 * http://www.opensource.org/licenses/mit-license.php
 */

// clean up database and config
require __DIR__.'/uninstall.php';

// init database

$addColumn = function($name, $ddl) use ($container) {
	$db  = $container['sly-persistence'];
	$pre = $db->getPrefix();

	$db->query('SHOW COLUMNS FROM '.$pre.'article WHERE Field = ?', array($name));

	if ($db->affectedRows() == 0) {
		$db->query('ALTER TABLE '.$pre.'article ADD COLUMN '.$name.' '.$ddl);
	}
};

$addColumn('wv24_fragment', 'VARCHAR(256) NOT NULL DEFAULT "" AFTER attributes');
$addColumn('wv24_skip', 'TINYINT(1) NOT NULL DEFAULT "0" AFTER wv24_fragment');

// reload global settings
// (makes it possible to access the settings and set some sensible default values)

$container['sly-dispatcher']->addListener('SLY_ADDON_POST_INSTALL', function() use ($container) {
	$request     = $container['sly-request'];
	$langService = $container['sly-service-language'];
	$provider    = $container['wv8-provider'];
	$manager     = $container['wv8-manager'];

	// reload settings
	$manager->refresh();

	// set default values
	$settings = $provider->getSettings('realurl2');
	$base     = substr($request->getBaseUrl(true, 'http'), 5); // cut off 'http:'
	$langs    = $langService->findAll();
	$defClang = $container['sly-config']->get('default_clang_id', key($langs));

	$fragmentValues  = array();
	$lowercaseValues = array();
	$cleanupValues   = array();

	foreach ($langs as $langID => $language) {
		// assume all languages have locales like 'de_DE'
		$locale = $language->getLocale();
		$locale = explode('_', $locale, 2);
		$locale = strtolower(reset($locale));

		if (empty($locale)) {
			$locale = 'lang'.$langID;
		}

		// set default values
		$fragmentValues[$langID]  = $base.($langID == $defClang ? '' : "/$locale");
		$lowercaseValues[$langID] = 1;
		$cleanupValues[$langID]   = 1;
	}

	$manager->storeValues($settings['realurl2/default_fragment'], $fragmentValues);
	$manager->storeValues($settings['realurl2/lowercase'], $lowercaseValues);
	$manager->storeValues($settings['realurl2/cleanup'], $cleanupValues);
});
